@@ -1,4 +1,4 @@
-// Copyright (c) 2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2022-2025 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -24,9 +24,8 @@
 #include "../config.hpp"
 #include "../functional.hpp"
 
-#include "../detail/radix_sort.hpp"
-
 #include "block_scan.hpp"
+#include "config.hpp"
 
 #include "detail/block_radix_rank_basic.hpp"
 #include "detail/block_radix_rank_match.hpp"
@@ -48,6 +47,8 @@ enum class block_radix_rank_algorithm
     match,
     /// \brief The default radix ranking algorithm.
     default_algorithm = basic,
+    /// \brief The placeholder for radix_sort default
+    default_for_radix_sort,
 };
 
 namespace detail
@@ -62,8 +63,11 @@ struct select_block_radix_rank_impl<block_radix_rank_algorithm::basic>
     template<unsigned int BlockSizeX,
              unsigned int RadixBits,
              unsigned int BlockSizeY,
-             unsigned int BlockSizeZ>
-    using type = block_radix_rank<BlockSizeX, RadixBits, false, BlockSizeY, BlockSizeZ>;
+             unsigned int BlockSizeZ,
+             block_padding_hint,
+             arch::wavefront::target TargetWaveSize>
+    using type
+        = block_radix_rank<BlockSizeX, RadixBits, false, BlockSizeY, BlockSizeZ, TargetWaveSize>;
 };
 
 template<>
@@ -72,18 +76,28 @@ struct select_block_radix_rank_impl<block_radix_rank_algorithm::basic_memoize>
     template<unsigned int BlockSizeX,
              unsigned int RadixBits,
              unsigned int BlockSizeY,
-             unsigned int BlockSizeZ>
-    using type = block_radix_rank<BlockSizeX, RadixBits, true, BlockSizeY, BlockSizeZ>;
+             unsigned int BlockSizeZ,
+             block_padding_hint,
+             arch::wavefront::target TargetWaveSize>
+    using type
+        = block_radix_rank<BlockSizeX, RadixBits, true, BlockSizeY, BlockSizeZ, TargetWaveSize>;
 };
 
 template<>
 struct select_block_radix_rank_impl<block_radix_rank_algorithm::match>
 {
-    template<unsigned int BlockSizeX,
-             unsigned int RadixBits,
-             unsigned int BlockSizeY,
-             unsigned int BlockSizeZ>
-    using type = block_radix_rank_match<BlockSizeX, RadixBits, BlockSizeY, BlockSizeZ>;
+    template<unsigned int            BlockSizeX,
+             unsigned int            RadixBits,
+             unsigned int            BlockSizeY,
+             unsigned int            BlockSizeZ,
+             block_padding_hint      PaddingHint,
+             arch::wavefront::target TargetWaveSize>
+    using type = block_radix_rank_match<BlockSizeX,
+                                        RadixBits,
+                                        BlockSizeY,
+                                        BlockSizeZ,
+                                        PaddingHint,
+                                        TargetWaveSize>;
 };
 } // namespace detail
 
@@ -92,12 +106,13 @@ struct select_block_radix_rank_impl<block_radix_rank_algorithm::match>
 /// associates each item with the index it would gain if the keys were sorted into an array,
 /// according to a radix comparison. Ranking is performed in a stable manner.
 ///
-/// \tparam BlockSizeX - the number of threads in a block's x dimension.
-/// \tparam RadixBits - the maximum number of radix digit bits that comparisons are performed by.
-/// \tparam MemoizeOuterScan - whether to cache digit counters in local memory. This omits loading
+/// \tparam BlockSizeX the number of threads in a block's x dimension.
+/// \tparam RadixBits the maximum number of radix digit bits that comparisons are performed by.
+/// \tparam MemoizeOuterScan whether to cache digit counters in local memory. This omits loading
 /// the same values from shared memory twice, at the expense of more register usage.
-/// \tparam BlockSizeY - the number of threads in a block's y dimension, defaults to 1.
-/// \tparam BlockSizeZ - the number of threads in a block's z dimension, defaults to 1.
+/// \tparam BlockSizeY the number of threads in a block's y dimension, defaults to 1.
+/// \tparam BlockSizeZ the number of threads in a block's z dimension, defaults to 1.
+/// \tparam PaddingHint a hint that decides when to use padding. May not always be applicable.
 ///
 /// \par Overview
 /// * Key type must be an arithmetic type (that is, an integral type or a floating point type).
@@ -137,17 +152,20 @@ struct select_block_radix_rank_impl<block_radix_rank_algorithm::match>
 /// \endcode
 template<unsigned int               BlockSizeX,
          unsigned int               RadixBits,
-         block_radix_rank_algorithm Algorithm  = block_radix_rank_algorithm::default_algorithm,
-         unsigned int               BlockSizeY = 1,
-         unsigned int               BlockSizeZ = 1>
+         block_radix_rank_algorithm Algorithm      = block_radix_rank_algorithm::default_algorithm,
+         unsigned int               BlockSizeY     = 1,
+         unsigned int               BlockSizeZ     = 1,
+         block_padding_hint         PaddingHint    = block_padding_hint::avoid_conflicts,
+         arch::wavefront::target    TargetWaveSize = arch::wavefront::get_target(),
+         typename Enabled                          = void>
 class block_radix_rank
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
-    : private detail::select_block_radix_rank_impl<
-          Algorithm>::template type<BlockSizeX, RadixBits, BlockSizeY, BlockSizeZ>
+    : private detail::select_block_radix_rank_impl<Algorithm>::
+          template type<BlockSizeX, RadixBits, BlockSizeY, BlockSizeZ, PaddingHint, TargetWaveSize>
 #endif
 {
-    using base_type = typename detail::select_block_radix_rank_impl<
-        Algorithm>::template type<BlockSizeX, RadixBits, BlockSizeY, BlockSizeZ>;
+    using base_type = typename detail::select_block_radix_rank_impl<Algorithm>::
+        template type<BlockSizeX, RadixBits, BlockSizeY, BlockSizeZ, PaddingHint, TargetWaveSize>;
 
 public:
     /// \brief The number of digits each thread will process.
@@ -165,14 +183,14 @@ public:
 
     /// \brief Perform ascending radix rank over keys partitioned across threads in a block.
     ///
-    /// \tparam Key - the key type.
-    /// \tparam ItemsPerThread - the number of items contributed by each thread in the block.
-    /// \param [in] keys - reference to an array of keys provided by a thread.
-    /// \param [out] ranks - reference to an array where the final ranks are written to.
-    /// \param [in] storage - reference to a temporary storage object of type \p storage_type.
-    /// \param [in] begin_bit - index of the first (least significant) bit used in key comparison.
+    /// \tparam Key the key type.
+    /// \tparam ItemsPerThread the number of items contributed by each thread in the block.
+    /// \param [in] keys reference to an array of keys provided by a thread.
+    /// \param [out] ranks reference to an array where the final ranks are written to.
+    /// \param [in] storage reference to a temporary storage object of type \p storage_type.
+    /// \param [in] begin_bit index of the first (least significant) bit used in key comparison.
     /// Must be in range <tt>[0; 8 * sizeof(Key))</tt>.
-    /// \param [in] pass_bits - [optional] the number of bits used in key comparison. Must be in
+    /// \param [in] pass_bits [optional] the number of bits used in key comparison. Must be in
     /// the range <tt>(0; RadixBits]</tt>. Default value: RadixBits.
     ///
     /// \par Storage reusage
@@ -220,13 +238,13 @@ public:
     /// * This overload does not accept storage argument. Required shared memory is allocated
     /// by the method itself.
     ///
-    /// \tparam Key - the key type.
-    /// \tparam ItemsPerThread - the number of items contributed by each thread in the block.
-    /// \param [in] keys - reference to an array of keys provided by a thread.
-    /// \param [out] ranks - reference to an array where the final ranks are written to.
-    /// \param [in] begin_bit - index of the first (least significant) bit used in key comparison.
+    /// \tparam Key the key type.
+    /// \tparam ItemsPerThread the number of items contributed by each thread in the block.
+    /// \param [in] keys reference to an array of keys provided by a thread.
+    /// \param [out] ranks reference to an array where the final ranks are written to.
+    /// \param [in] begin_bit index of the first (least significant) bit used in key comparison.
     /// Must be in range <tt>[0; 8 * sizeof(Key))</tt>.
-    /// \param [in] pass_bits - [optional] the number of bits used in key comparison. Must be in
+    /// \param [in] pass_bits [optional] the number of bits used in key comparison. Must be in
     /// the range <tt>(0; RadixBits]</tt>. Default value: RadixBits.
     template<typename Key, unsigned ItemsPerThread>
     ROCPRIM_DEVICE void rank_keys(const Key (&keys)[ItemsPerThread],
@@ -241,13 +259,13 @@ public:
     /// \brief Perform ascending radix rank over bit keys partitioned across threads in a block.
     /// This overload accepts a callback used to extract the radix digit from a key.
     ///
-    /// \tparam ItemsPerThread - the number of items contributed by each thread in the block.
-    /// \param [in] keys - reference to an array of keys provided by a thread.
-    /// \param [out] ranks - reference to an array where the final ranks are written to.
-    /// \param [in] storage - reference to a temporary storage object of type \p storage_type.
-    /// \param [in] begin_bit - index of the first (least significant) bit used in key comparison.
+    /// \tparam ItemsPerThread the number of items contributed by each thread in the block.
+    /// \param [in] keys reference to an array of keys provided by a thread.
+    /// \param [out] ranks reference to an array where the final ranks are written to.
+    /// \param [in] storage reference to a temporary storage object of type \p storage_type.
+    /// \param [in] begin_bit index of the first (least significant) bit used in key comparison.
     /// Must be in range <tt>[0; 8 * sizeof(Key))</tt>.
-    /// \param [in] pass_bits - [optional] the number of bits used in key comparison. Must be in
+    /// \param [in] pass_bits [optional] the number of bits used in key comparison. Must be in
     /// the range <tt>(0; RadixBits]</tt>. Default value: RadixBits.
     ///
     /// \par Storage reusage
@@ -295,13 +313,13 @@ public:
     /// * This overload does not accept storage argument. Required shared memory is allocated
     /// by the method itself.
     ///
-    /// \tparam Key - the key type.
-    /// \tparam ItemsPerThread - the number of items contributed by each thread in the block.
-    /// \param [in] keys - reference to an array of keys provided by a thread.
-    /// \param [out] ranks - reference to an array where the final ranks are written to.
-    /// \param [in] begin_bit - index of the first (least significant) bit used in key comparison.
+    /// \tparam Key the key type.
+    /// \tparam ItemsPerThread the number of items contributed by each thread in the block.
+    /// \param [in] keys reference to an array of keys provided by a thread.
+    /// \param [out] ranks reference to an array where the final ranks are written to.
+    /// \param [in] begin_bit index of the first (least significant) bit used in key comparison.
     /// Must be in range <tt>[0; 8 * sizeof(Key))</tt>.
-    /// \param [in] pass_bits - [optional] the number of bits used in key comparison. Must be in
+    /// \param [in] pass_bits [optional] the number of bits used in key comparison. Must be in
     /// the range <tt>(0; RadixBits]</tt>. Default value: RadixBits.
     template<typename Key, unsigned ItemsPerThread>
     ROCPRIM_DEVICE void rank_keys_desc(const Key (&keys)[ItemsPerThread],
@@ -316,14 +334,14 @@ public:
     /// \brief Perform ascending radix rank over bit keys partitioned across threads in a block.
     /// This overload accepts a callback used to extract the radix digit from a key.
     ///
-    /// \tparam Key - the key type.
-    /// \tparam ItemsPerThread - the number of items contributed by each thread in the block.
-    /// \tparam DigitExtractor - type of the unary function object used to extract a digit from
+    /// \tparam Key the key type.
+    /// \tparam ItemsPerThread the number of items contributed by each thread in the block.
+    /// \tparam DigitExtractor type of the unary function object used to extract a digit from
     /// a key.
-    /// \param [in] keys - reference to an array of keys provided by a thread.
-    /// \param [out] ranks - reference to an array where the final ranks are written to.
-    /// \param [in] storage - reference to a temporary storage object of type \p storage_type.
-    /// \param [in] digit_extractor - function object used to convert a key to a digit.
+    /// \param [in] keys reference to an array of keys provided by a thread.
+    /// \param [out] ranks reference to an array where the final ranks are written to.
+    /// \param [in] storage reference to a temporary storage object of type \p storage_type.
+    /// \param [in] digit_extractor function object used to convert a key to a digit.
     /// The signature of the \p digit_extractor should be equivalent to the following:
     /// <tt>unsigned int f(const Key &key);</tt>. The signature does not need to have
     /// <tt>const &</tt>, but function object must not modify the objects passed to it.
@@ -378,13 +396,13 @@ public:
     /// * This overload does not accept storage argument. Required shared memory is allocated
     /// by the method itself.
     ///
-    /// \tparam Key - the key type.
-    /// \tparam ItemsPerThread - the number of items contributed by each thread in the block.
-    /// \tparam DigitExtractor - type of the unary function object used to extract a digit from
+    /// \tparam Key the key type.
+    /// \tparam ItemsPerThread the number of items contributed by each thread in the block.
+    /// \tparam DigitExtractor type of the unary function object used to extract a digit from
     /// a key.
-    /// \param [in] keys - reference to an array of keys provided by a thread.
-    /// \param [out] ranks - reference to an array where the final ranks are written to.
-    /// \param [in] digit_extractor - function object used to convert a key to a digit.
+    /// \param [in] keys reference to an array of keys provided by a thread.
+    /// \param [out] ranks reference to an array where the final ranks are written to.
+    /// \param [in] digit_extractor function object used to convert a key to a digit.
     /// The signature of the \p digit_extractor should be equivalent to the following:
     /// <tt>unsigned int f(const Key &key);</tt>. The signature does not need to have
     /// <tt>const &</tt>, but function object must not modify the objects passed to it.
@@ -402,14 +420,14 @@ public:
     /// \brief Perform descending radix rank over bit keys partitioned across threads in a block.
     /// This overload accepts a callback used to extract the radix digit from a key.
     ///
-    /// \tparam Key - the key type.
-    /// \tparam ItemsPerThread - the number of items contributed by each thread in the block.
-    /// \tparam DigitExtractor - type of the unary function object used to extract a digit from
+    /// \tparam Key the key type.
+    /// \tparam ItemsPerThread the number of items contributed by each thread in the block.
+    /// \tparam DigitExtractor type of the unary function object used to extract a digit from
     /// a key.
-    /// \param [in] keys - reference to an array of keys provided by a thread.
-    /// \param [out] ranks - reference to an array where the final ranks are written to.
-    /// \param [in] storage - reference to a temporary storage object of type \p storage_type.
-    /// \param [in] digit_extractor - function object used to convert a key to a digit.
+    /// \param [in] keys reference to an array of keys provided by a thread.
+    /// \param [out] ranks reference to an array where the final ranks are written to.
+    /// \param [in] storage reference to a temporary storage object of type \p storage_type.
+    /// \param [in] digit_extractor function object used to convert a key to a digit.
     /// The signature of the \p digit_extractor should be equivalent to the following:
     /// <tt>unsigned int f(const Key &key);</tt>. The signature does not need to have
     /// <tt>const &</tt>, but function object must not modify the objects passed to it.
@@ -464,13 +482,13 @@ public:
     /// * This overload does not accept storage argument. Required shared memory is allocated
     /// by the method itself.
     ///
-    /// \tparam Key - the key type.
-    /// \tparam ItemsPerThread - the number of items contributed by each thread in the block.
-    /// \tparam DigitExtractor - type of the unary function object used to extract a digit from
+    /// \tparam Key the key type.
+    /// \tparam ItemsPerThread the number of items contributed by each thread in the block.
+    /// \tparam DigitExtractor type of the unary function object used to extract a digit from
     /// a key.
-    /// \param [in] keys - reference to an array of keys provided by a thread.
-    /// \param [out] ranks - reference to an array where the final ranks are written to.
-    /// \param [in] digit_extractor - function object used to convert a key to a digit.
+    /// \param [in] keys reference to an array of keys provided by a thread.
+    /// \param [out] ranks reference to an array where the final ranks are written to.
+    /// \param [in] digit_extractor function object used to convert a key to a digit.
     /// The signature of the \p digit_extractor should be equivalent to the following:
     /// <tt>unsinged int f(const Key &key);</tt>. The signature does not need to have
     /// <tt>const &</tt>, but function object must not modify the objects passed to it.
@@ -489,23 +507,23 @@ public:
     /// This overload accepts a callback used to extract the radix digit from a key, and provides
     /// the counts of each digit and a prefix scan thereof in a blocked arrangement.
     ///
-    /// \tparam Key - the key type.
-    /// \tparam ItemsPerThread - the number of items contributed by each thread in the block.
-    /// \tparam DigitExtractor - type of the unary function object used to extract a digit from
+    /// \tparam Key the key type.
+    /// \tparam ItemsPerThread the number of items contributed by each thread in the block.
+    /// \tparam DigitExtractor type of the unary function object used to extract a digit from
     /// a key.
-    /// \param [in] keys - reference to an array of keys provided by a thread. Keys are expected in
+    /// \param [in] keys reference to an array of keys provided by a thread. Keys are expected in
     /// warp-striped arrangement.
-    /// \param [out] ranks - reference to an array where the final ranks are written to. Ranks are
+    /// \param [out] ranks reference to an array where the final ranks are written to. Ranks are
     /// provided in warp-striped arrangement.
-    /// \param [in] storage - reference to a temporary storage object of type \p storage_type.
-    /// \param [in] digit_extractor - function object used to convert a key to a digit.
+    /// \param [in] storage reference to a temporary storage object of type \p storage_type.
+    /// \param [in] digit_extractor function object used to convert a key to a digit.
     /// The signature of the \p digit_extractor should be equivalent to the following:
     /// <tt>unsigned int f(const Key &key);</tt>. The signature does not need to have
     /// <tt>const &</tt>, but function object must not modify the objects passed to it.
     /// This function will be used during ranking to extract the digit that indicates
     /// the key's value. Values return by this function object must be in range [0; 1 << RadixBits).
-    /// \param [in] prefix - An exclusive prefix scan of the counts per digit.
-    /// \param [in] counts - The number of keys with a particular digit in the input, per digit.
+    /// \param [in] prefix An exclusive prefix scan of the counts per digit.
+    /// \param [in] counts The number of keys with a particular digit in the input, per digit.
     ///
     /// \par Storage reusage
     /// A synchronization barrier should be placed before \p storage is reused
@@ -549,6 +567,63 @@ public:
     }
 };
 
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
+template<unsigned int               BlockSizeX,
+         unsigned int               RadixBits,
+         block_radix_rank_algorithm Algorithm,
+         unsigned int               BlockSizeY,
+         unsigned int               BlockSizeZ,
+         block_padding_hint         PaddingHint>
+class block_radix_rank<BlockSizeX,
+                       RadixBits,
+                       Algorithm,
+                       BlockSizeY,
+                       BlockSizeZ,
+                       PaddingHint,
+                       arch::wavefront::target::dynamic>
+{
+private:
+    using block_radix_rank_wave32 = block_radix_rank<BlockSizeX,
+                                                     RadixBits,
+                                                     Algorithm,
+                                                     BlockSizeY,
+                                                     BlockSizeZ,
+                                                     PaddingHint,
+                                                     arch::wavefront::target::size32>;
+    using block_radix_rank_wave64 = block_radix_rank<BlockSizeX,
+                                                     RadixBits,
+                                                     Algorithm,
+                                                     BlockSizeY,
+                                                     BlockSizeZ,
+                                                     PaddingHint,
+                                                     arch::wavefront::target::size64>;
+
+    using dispatch = detail::dispatch_wave_size<block_radix_rank_wave32, block_radix_rank_wave64>;
+
+public:
+    static_assert(block_radix_rank_wave32::digits_per_thread
+                      == block_radix_rank_wave64::digits_per_thread,
+                  "digits_per_thread is not the same for wavefront size 32 and 64!");
+    static constexpr unsigned int digits_per_thread = block_radix_rank_wave32::digits_per_thread;
+
+    using storage_type = typename dispatch::storage_type;
+
+    template<typename... Args>
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    auto rank_keys(Args&&... args)
+    {
+        dispatch{}([](auto impl, auto&&... args) { impl.rank_keys(args...); }, args...);
+    }
+
+    template<typename... Args>
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    auto rank_keys_desc(Args&&... args)
+    {
+        dispatch{}([](auto impl, auto&&... args) { impl.rank_keys_desc(args...); }, args...);
+    }
+};
+#endif // DOXYGEN_SHOULD_SKIP_THIS
+
 END_ROCPRIM_NAMESPACE
 
 /// @}
