@@ -155,22 +155,6 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE void
     block_store_key().store(keys, sample_buffer, size, storage.store);
 }
 
-template<class Config, class KeysIterator, class BinaryFunction>
-inline hipError_t launch_block_sort(detail::target_arch arch,
-                                    KeysIterator        keys,
-                                    const unsigned int  size,
-                                    BinaryFunction      compare_function,
-                                    dim3                grid,
-                                    dim3                block,
-                                    size_t              shmem,
-                                    hipStream_t         stream)
-{
-    auto kernel = [=](auto arch_config)
-    { block_sort_kernel_impl<decltype(arch_config)>(keys, size, compare_function); };
-
-    return execute_launch_plan<Config>(arch, kernel, grid, block, shmem, stream);
-}
-
 template<class ArchConfig, class KeysIterator, class BinaryFunction>
 ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE void find_splitters_kernel_impl(
                                KeysIterator                                             keys,
@@ -213,31 +197,6 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE void find_splitters_kernel_impl(
     equality_buckets[idx] = equality_bucket;
 }
 
-template<class Config, class KeysIterator, class BinaryFunction>
-inline hipError_t
-    launch_find_splitters(detail::target_arch                                      arch,
-                          KeysIterator                                             keys,
-                          typename std::iterator_traits<KeysIterator>::value_type* tree,
-                          bool*                                                    equality_buckets,
-                          const unsigned int                                       size,
-                          BinaryFunction                                           compare_function,
-                          dim3                                                     grid,
-                          dim3                                                     block,
-                          size_t                                                   shmem,
-                          hipStream_t                                              stream)
-{
-    auto kernel = [=](auto arch_config)
-    {
-        find_splitters_kernel_impl<decltype(arch_config)>(keys,
-                                                          tree,
-                                                          equality_buckets,
-                                                          size,
-                                                          compare_function);
-    };
-
-    return execute_launch_plan<Config>(arch, kernel, grid, block, shmem, stream);
-}
-
 template<class ArchConfig, class KeysIterator, class BinaryFunction>
 ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE void
     count_bucket_sizes_kernel_impl(KeysIterator                                             keys,
@@ -342,33 +301,6 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE void
     }
 }
 
-template<class Config, class KeysIterator, class BinaryFunction>
-inline hipError_t
-    launch_count_bucket_sizes(detail::target_arch                                      arch,
-                              KeysIterator                                             keys,
-                              typename std::iterator_traits<KeysIterator>::value_type* tree,
-                              const unsigned int                                       size,
-                              unsigned int*                                            buckets,
-                              bool*          equality_buckets,
-                              BinaryFunction compare_function,
-                              dim3           grid,
-                              dim3           block,
-                              size_t         shmem,
-                              hipStream_t    stream)
-{
-    auto kernel = [=](auto arch_config)
-    {
-        count_bucket_sizes_kernel_impl<decltype(arch_config)>(keys,
-                                                              tree,
-                                                              size,
-                                                              buckets,
-                                                              equality_buckets,
-                                                              compare_function);
-    };
-
-    return execute_launch_plan<Config>(arch, kernel, grid, block, shmem, stream);
-}
-
 template<class ArchConfig>
 ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE void
     find_nth_element_bucket_kernel_impl(unsigned int*                      buckets,
@@ -415,29 +347,6 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE void
     }
 }
 
-template<class Config>
-inline hipError_t launch_find_nth_element_bucket(detail::target_arch          arch,
-                                                 unsigned int*                buckets,
-                                                 n_th_element_iteration_data* nth_element_data,
-                                                 bool*                        equality_buckets,
-                                                 const unsigned int           rank,
-                                                 dim3                         grid,
-                                                 dim3                         block,
-                                                 size_t                       shmem,
-                                                 hipStream_t                  stream)
-
-{
-    auto kernel = [=](auto arch_config)
-    {
-        find_nth_element_bucket_kernel_impl<decltype(arch_config)>(buckets,
-                                                                   nth_element_data,
-                                                                   equality_buckets,
-                                                                   rank);
-    };
-
-    return execute_launch_plan<Config>(arch, kernel, grid, block, shmem, stream);
-}
-
 template<class ArchConfig, unsigned int NumPartitions, class KeysIterator, class BinaryFunction, class WrappedBlockId>
 ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE void
     copy_buckets_kernel_impl(KeysIterator                                             keys,
@@ -632,43 +541,6 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE void
     }
 }
 
-template<class Config,
-         unsigned int NumPartitions,
-         class KeysIterator,
-         class BinaryFunction,
-         class WrappedBlockId>
-inline hipError_t
-    launch_copy_buckets(detail::target_arch                                      arch,
-                        KeysIterator                                             keys,
-                        typename std::iterator_traits<KeysIterator>::value_type* tree,
-                        const unsigned int                                       size,
-                        nth_element_onesweep_lookback_state*                     lookback_states,
-                        n_th_element_iteration_data*                             nth_element_data,
-                        typename std::iterator_traits<KeysIterator>::value_type* keys_buffer,
-                        bool*                                                    equality_buckets,
-                        BinaryFunction                                           compare_function,
-                        WrappedBlockId                                           ordered_bid,
-                        dim3                                                     grid,
-                        dim3                                                     block,
-                        size_t                                                   shmem,
-                        hipStream_t                                              stream)
-{
-    auto kernel = [=](auto arch_config)
-    {
-        copy_buckets_kernel_impl<decltype(arch_config), NumPartitions>(keys,
-                                                                       tree,
-                                                                       size,
-                                                                       lookback_states,
-                                                                       nth_element_data,
-                                                                       keys_buffer,
-                                                                       equality_buckets,
-                                                                       compare_function,
-                                                                       ordered_bid);
-    };
-
-    return execute_launch_plan<Config>(arch, kernel, grid, block, shmem, stream);
-}
-
 template<class Config,
          unsigned int NumPartitions,
          class KeysIterator,
@@ -676,7 +548,7 @@ template<class Config,
          class WrappedBlockId>
 ROCPRIM_INLINE
 hipError_t
-    nth_element_keys_impl(detail::target_arch                                      arch,
+    nth_element_keys_impl(detail::target_arch                                      target_arch,
                           KeysIterator                                             keys,
                           typename std::iterator_traits<KeysIterator>::value_type* keys_buffer,
                           typename std::iterator_traits<KeysIterator>::value_type* tree,
@@ -740,47 +612,60 @@ hipError_t
         ROCPRIM_RETURN_ON_ERROR(ordered_bid.reset_from_host(stream));
 
         start_timer();
-        ROCPRIM_RETURN_ON_ERROR(launch_find_splitters<Config>(arch,
-                                                              keys,
+        auto find_splitters_kernel = [=](auto arch_config)
+        {
+            find_splitters_kernel_impl<decltype(arch_config)>(keys,
                                                               tree,
                                                               equality_buckets,
                                                               size,
-                                                              compare_function,
-                                                              1,
-                                                              num_splitters,
-                                                              0,
-                                                              stream));
+                                                              compare_function);
+        };
+        ROCPRIM_RETURN_ON_ERROR(execute_launch_plan<Config>(target_arch,
+                                                            find_splitters_kernel,
+                                                            1,
+                                                            num_splitters,
+                                                            0,
+                                                            stream));
         ROCPRIM_DETAIL_HIP_SYNC_AND_RETURN_ON_ERROR("find_splitters_kernel", size, start);
 
         start_timer();
-        ROCPRIM_RETURN_ON_ERROR(launch_count_bucket_sizes<Config>(arch,
-                                                                  keys,
+        auto count_bucket_sizes_kernel = [=](auto arch_config)
+        {
+            count_bucket_sizes_kernel_impl<decltype(arch_config)>(keys,
                                                                   tree,
                                                                   size,
                                                                   buckets,
                                                                   equality_buckets,
-                                                                  compare_function,
-                                                                  num_blocks,
-                                                                  num_threads_per_block,
-                                                                  0,
-                                                                  stream));
+                                                                  compare_function);
+        };
+        ROCPRIM_RETURN_ON_ERROR(execute_launch_plan<Config>(target_arch,
+                                                            count_bucket_sizes_kernel,
+                                                            num_blocks,
+                                                            num_threads_per_block,
+                                                            0,
+                                                            stream));
         ROCPRIM_DETAIL_HIP_SYNC_AND_RETURN_ON_ERROR("count_bucket_sizes_kernel", size, start);
 
         start_timer();
-        ROCPRIM_RETURN_ON_ERROR(launch_find_nth_element_bucket<Config>(arch,
-                                                                       buckets,
+        auto find_nth_element_bucket_kernel = [=](auto arch_config)
+        {
+            find_nth_element_bucket_kernel_impl<decltype(arch_config)>(buckets,
                                                                        nth_element_data,
                                                                        equality_buckets,
-                                                                       rank,
-                                                                       1,
-                                                                       num_buckets,
-                                                                       0,
-                                                                       stream));
+                                                                       rank);
+        };
+        ROCPRIM_RETURN_ON_ERROR(execute_launch_plan<Config>(target_arch,
+                                                            find_nth_element_bucket_kernel,
+                                                            1,
+                                                            num_buckets,
+                                                            0,
+                                                            stream));
         ROCPRIM_DETAIL_HIP_SYNC_AND_RETURN_ON_ERROR("find_nth_element_bucket_kernel", size, start);
 
         start_timer();
-        ROCPRIM_RETURN_ON_ERROR(launch_copy_buckets<Config, num_partitions>(arch,
-                                                                            keys,
+        auto copy_buckets_kernel = [=](auto arch_config)
+        {
+            copy_buckets_kernel_impl<decltype(arch_config), num_partitions>(keys,
                                                                             tree,
                                                                             size,
                                                                             lookback_states,
@@ -788,11 +673,14 @@ hipError_t
                                                                             keys_buffer,
                                                                             equality_buckets,
                                                                             compare_function,
-                                                                            ordered_bid,
-                                                                            num_blocks,
-                                                                            num_threads_per_block,
-                                                                            0,
-                                                                            stream));
+                                                                            ordered_bid);
+        };
+        ROCPRIM_RETURN_ON_ERROR(execute_launch_plan<Config>(target_arch,
+                                                            copy_buckets_kernel,
+                                                            num_blocks,
+                                                            num_threads_per_block,
+                                                            0,
+                                                            stream));
         ROCPRIM_DETAIL_HIP_SYNC_AND_RETURN_ON_ERROR("copy_buckets_kernel", size, start);
 
         // Copy the results in keys_buffer back to the keys
@@ -829,14 +717,15 @@ hipError_t
     }
 
     start_timer();
-    ROCPRIM_RETURN_ON_ERROR(launch_block_sort<Config>(arch,
-                                                      keys,
-                                                      size,
-                                                      compare_function,
-                                                      1,
-                                                      stop_recursion_size,
-                                                      0,
-                                                      stream));
+    auto block_sort_kernel = [=](auto arch_config)
+    { block_sort_kernel_impl<decltype(arch_config)>(keys, size, compare_function); };
+
+    ROCPRIM_RETURN_ON_ERROR(execute_launch_plan<Config>(target_arch,
+                                                        block_sort_kernel,
+                                                        1,
+                                                        stop_recursion_size,
+                                                        0,
+                                                        stream));
     ROCPRIM_DETAIL_HIP_SYNC_AND_RETURN_ON_ERROR("kernel_block_sort", size, start);
     return hipSuccess;
 }
