@@ -72,40 +72,6 @@ struct transform_op_t
     }
 };
 
-template<class Config,
-         bool Exclusive,
-         class ResultType,
-         class InputIterator,
-         class OutputIterator,
-         class OffsetIterator,
-         class InitValueType,
-         class BinaryFunction>
-inline hipError_t launch_segmented_scan(detail::target_arch arch,
-                                        InputIterator       input,
-                                        OutputIterator      output,
-                                        OffsetIterator      begin_offsets,
-                                        OffsetIterator      end_offsets,
-                                        InitValueType       initial_value,
-                                        BinaryFunction      scan_op,
-                                        dim3                grid,
-                                        dim3                block,
-                                        size_t              shmem,
-                                        hipStream_t         stream)
-{
-    auto kernel = [=](auto arch_config)
-    {
-        segmented_scan<decltype(arch_config), Exclusive, ResultType>(
-            input,
-            output,
-            begin_offsets,
-            end_offsets,
-            static_cast<ResultType>(initial_value),
-            scan_op);
-    };
-
-    return execute_launch_plan<Config>(arch, kernel, grid, block, shmem, stream);
-}
-
 template<bool Exclusive,
          class Config,
          class InputIterator,
@@ -153,18 +119,26 @@ inline hipError_t segmented_scan_impl(void*               temporary_storage,
 
     std::chrono::steady_clock::time_point start;
     if(debug_synchronous)
+    {
         start = std::chrono::steady_clock::now();
-    ROCPRIM_RETURN_ON_ERROR(launch_segmented_scan<config, Exclusive, result_type>(target_arch,
-                                                                                  input,
-                                                                                  output,
-                                                                                  begin_offsets,
-                                                                                  end_offsets,
-                                                                                  initial_value,
-                                                                                  scan_op,
-                                                                                  dim3(segments),
-                                                                                  dim3(block_size),
-                                                                                  0,
-                                                                                  stream));
+    }
+    auto segmented_scan_kernel = [=](auto arch_config)
+    {
+        segmented_scan<decltype(arch_config), Exclusive, result_type>(
+            input,
+            output,
+            begin_offsets,
+            end_offsets,
+            static_cast<result_type>(initial_value),
+            scan_op);
+    };
+
+    ROCPRIM_RETURN_ON_ERROR(execute_launch_plan<config>(target_arch,
+                                                        segmented_scan_kernel,
+                                                        dim3(segments),
+                                                        dim3(block_size),
+                                                        0,
+                                                        stream));
     ROCPRIM_DETAIL_HIP_SYNC_AND_RETURN_ON_ERROR("segmented_scan", segments, start);
     return hipSuccess;
 }
