@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2025 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -35,6 +35,8 @@
 #include "types/tuple.hpp"
 #include "types/uninitialized_array.hpp"
 
+#include "intrinsics/arch.hpp"
+
 /// \addtogroup utilsmodule
 /// @{
 
@@ -54,7 +56,8 @@ struct make_vector_type
 
 /// \brief Empty type used as a placeholder, usually used to flag that given
 /// template parameter should not be used.
-struct empty_type {};
+struct empty_type
+{};
 
 /// \brief A decomposer that must be passed to the radix sort algorithms when
 /// sorting keys that are arithmetic types.
@@ -74,14 +77,19 @@ using bfloat16 = ::hip_bfloat16;
 
 /// \brief The lane_mask_type is an integer that contains one bit per thread.
 ///
-/// The total number of bits is equal to the total number of threads in a
-/// warp. Used to for warp-level operations.
-/// \note This is defined only on the device side, see `ROCPRIM_WAVEFRONT_SIZE` for details.
-#if ROCPRIM_WAVEFRONT_SIZE == 32
-using lane_mask_type = unsigned int;
-#elif ROCPRIM_WAVEFRONT_SIZE == 64
-using lane_mask_type = unsigned long long int;
-#endif
+/// When targeting AMDGCN, the total number of bits is equal to the total
+/// number of threads in a warp. Used for warp-level operations. When
+/// targeting SPIR-V, it assumes 64 threads per warp.
+///
+/// \note When called on the host, assumes 64-bit wide masks.
+///
+/// \note When targeting SPIR-V, this type will be 64-bit wide. Extra
+/// precaution must be taken as the number of bits in this type is not
+/// always the same as the number of lanes.
+using lane_mask_type = std::conditional_t<::rocprim::arch::wavefront::get_target()
+                                              == ::rocprim::arch::wavefront::target::size32,
+                                          unsigned int,
+                                          unsigned long long int>;
 
 /// \brief Native half-precision floating point type
 using native_half = _Float16;
